@@ -3,9 +3,9 @@
 ;; Copyright (C) 2023  Free Software Foundation, Inc.
 
 ;; Author: Mohamed Suliman <sulimanm@tcd.ie>
-;; Version: 1.1.1
+;; Version: 1.2.0
 ;; URL: https://github.com/namilus/denote-menu
-;; Package-Requires: ((emacs "28.1") (denote "1.2.0"))
+;; Package-Requires: ((emacs "28.1") (denote "2.0.0"))
 
 ;; This file is NOT part of GNU Emacs.
 
@@ -44,6 +44,7 @@
 (require 'tabulated-list)
 (require 'denote)
 (require 'dired)
+(require 'seq)
 
 (defgroup denote-menu ()
   "View Denote files"
@@ -54,18 +55,21 @@
   :type 'number
   :group 'denote-menu)
 
+(defcustom denote-menu-signature-column-width 10
+  "Width for the date column."
+  :type 'number
+  :group 'denote-menu)
+
 (defcustom denote-menu-title-column-width 85
   "Width for the title column."
   :type 'number
   :group 'denote-menu)
 
-
 (defcustom denote-menu-keywords-column-width 30
   "Width for the keywords column."
   :type 'number
   :group 'denote-menu)
 
-
 (defcustom denote-menu-action (lambda (path) (find-file path))
   "Function to execute when a denote file button action is
 invoked. Takes a single argument which is the path of the
@@ -78,9 +82,13 @@ denote file corresponding to the button."
   :type 'string
   :group 'denote-menu)
 
-
 (defcustom denote-menu-show-file-type t
-  "Whether to show the denote file type"
+  "Whether to show the denote file type."
+  :type 'boolean
+  :group 'denote-menu)
+
+(defcustom denote-menu-show-file-signature nil
+  "Whether to show the denote file signature."
   :type 'boolean
   :group 'denote-menu)
 
@@ -91,13 +99,17 @@ denote file corresponding to the button."
 (defun denote-menu-list-notes ()
   "Display list of Denote files in variable `denote-directory'."
   (interactive)
-  (let ((buffer (get-buffer-create "*Denote*")))
-    (with-current-buffer buffer
-      (setq buffer-file-coding-system 'utf-8)
-      (setq denote-menu-current-regex denote-menu-initial-regex)
-      (denote-menu-mode))
+  ;; kill any existing *Denote* buffer
+  (let ((denote-menu-buffer-name (format "*Denote %s*" denote-directory)))
+    (when (get-buffer  denote-menu-buffer-name)
+      (kill-buffer denote-menu-buffer-name))
+    (let ((buffer (get-buffer-create denote-menu-buffer-name)))
+      (with-current-buffer buffer
+        (setq buffer-file-coding-system 'utf-8)
+        (setq denote-menu-current-regex denote-menu-initial-regex)
+        (denote-menu-mode))
     
-    (pop-to-buffer-same-window buffer)))
+      (pop-to-buffer-same-window buffer))))
 
 (defalias 'list-denotes 'denote-menu-list-notes
   "Alias of `denote-menu-list-notes' command.")
@@ -162,13 +174,19 @@ PATH and appending the filename extension."
         (extension (file-name-extension path)))
     (format "%s-%s" path-identifier extension)))
 
-
 (defun denote-menu--path-to-entry (path)
   "Convert PATH to an entry matching the form of `tabulated-list-entries'."
-  `(,(denote-menu--path-to-unique-identifier path)
-    [(,(denote-menu-date path) . (action ,(lambda (button) (funcall denote-menu-action path))))
-     ,(denote-menu-title path)
-     ,(propertize (format "%s" (denote-extract-keywords-from-path path)) 'face 'italic)]))
+  (if denote-menu-show-file-signature
+      `(,(denote-menu--path-to-unique-identifier path)
+        [(,(denote-menu-date path) . (action ,(lambda (button) (funcall denote-menu-action path))))
+         ,(denote-menu-signature path)
+         ,(denote-menu-title path)
+         ,(propertize (format "%s" (denote-extract-keywords-from-path path)) 'face 'italic)])
+
+    `(,(denote-menu--path-to-unique-identifier path)
+        [(,(denote-menu-date path) . (action ,(lambda (button) (funcall denote-menu-action path))))
+         ,(denote-menu-title path)
+         ,(propertize (format "%s" (denote-extract-keywords-from-path path)) 'face 'italic)])))
   
 (defun denote-menu-date (path)
   "Return human readable date from denote PATH identifier."
@@ -184,6 +202,12 @@ PATH and appending the filename extension."
                   
     (format "%s-%s-%s %s:%s" year month day hour seconds)))
 
+(defun denote-menu-signature (path)
+  "Return file signature from denote PATH identifier."
+  (let ((signature (denote-retrieve-filename-signature path)))
+    (if signature
+        signature
+      (propertize " " 'face 'font-lock-comment-face))))
 
 (defun denote-menu-type (path)
   "Return file type of PATH"
@@ -220,9 +244,31 @@ Revert the *Denotes* buffer to include only the matching entries."
   "Prompt for KEYWORDS and filters the list accordingly.
 When called from Lisp, KEYWORDS is a list of strings."
   (interactive (list (denote-keywords-prompt)))
-  (let ((regex (concat "\\(" (mapconcat (lambda (keyword) (format "_%s" keyword)) keywords "\\|") "\\)")))
+  (let ((regex (denote-menu--keywords-to-regex keywords)))
     (setq denote-menu-current-regex regex)
     (denote-menu-update-entries)))
+
+(defun denote-menu--keywords-to-regex (keywords)
+  "Converts KEYWORDS into a regex that matches denote files that
+contain at least one of the keywords."
+  (concat "\\(" (mapconcat (lambda (keyword) (format "_%s" keyword)) keywords "\\|") "\\)"))
+
+(defun denote-menu-filter-out-keyword (keywords)
+  "Prompt for KEYWORDS and filters out of the list those denote
+files that contain one of the keywords. When called from Lisp,
+ KEYWORDS is a list of strings."
+  (interactive (list (denote-keywords-prompt)))
+  ;; need to get a list of the denote files that do not include the
+  ;; keywords and set tabulated entries to be those.
+  (let* ((regex (denote-menu--keywords-to-regex keywords))
+        (non-matching-files (seq-filter
+                             (lambda (f)
+                               (not (string-match-p regex (denote-get-file-name-relative-to-denote-directory f))))
+                             (denote-menu--entries-to-paths))))
+    (setq tabulated-list-entries
+          (lambda ()
+            (mapcar #'denote-menu--path-to-entry non-matching-files))))
+  (revert-buffer))
     
 (defun denote-menu-clear-filters ()
   "Reset filters to `denote-menu-initial-regex' and update buffer."
@@ -249,9 +295,15 @@ files."
 (define-derived-mode denote-menu-mode tabulated-list-mode "Denote Menu"
   "Major mode for browsing a list of Denote files."
   :interactive nil
-  (setq tabulated-list-format `[("Date" ,denote-menu-date-column-width t)
-                                ("Title" ,denote-menu-title-column-width nil)
-                                ("Keywords" ,denote-menu-keywords-column-width nil)])
+  (if denote-menu-show-file-signature
+      (setq tabulated-list-format `[("Date" ,denote-menu-date-column-width t)
+                                    ("Signature" ,denote-menu-signature-column-width nil)
+                                    ("Title" ,denote-menu-title-column-width nil)
+                                    ("Keywords" ,denote-menu-keywords-column-width nil)])
+
+    (setq tabulated-list-format `[("Date" ,denote-menu-date-column-width t)
+                                  ("Title" ,denote-menu-title-column-width nil)
+                                  ("Keywords" ,denote-menu-keywords-column-width nil)]))
 
   (denote-menu-update-entries)
   (setq tabulated-list-sort-key '("Date" . t))
